@@ -5,7 +5,7 @@
     Contains SphinxError and a few subclasses (in an extra module to avoid
     circular import problems).
 
-    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -47,12 +47,19 @@ class ApplicationError(SphinxError):
 
 class ExtensionError(SphinxError):
     """Extension error."""
-    category = 'Extension error'
 
-    def __init__(self, message: str, orig_exc: Exception = None) -> None:
+    def __init__(self, message: str, orig_exc: Exception = None, modname: str = None) -> None:
         super().__init__(message)
         self.message = message
         self.orig_exc = orig_exc
+        self.modname = modname
+
+    @property
+    def category(self) -> str:  # type: ignore
+        if self.modname:
+            return 'Extension error (%s)' % self.modname
+        else:
+            return 'Extension error'
 
     def __repr__(self) -> str:
         if self.orig_exc:
@@ -122,5 +129,5 @@ class NoUri(Exception):
 
 
 class FiletypeNotFoundError(Exception):
-    "Raised by get_filetype() if a filename matches no source suffix."
+    """Raised by get_filetype() if a filename matches no source suffix."""
     pass
