@@ -103,10 +103,10 @@ class _TranslationProxy(UserString):
             return '<%s broken>' % self.__class__.__name__
 
 
-translators = defaultdict(NullTranslations)  # type: Dict[Tuple[str, str], NullTranslations]
+translators: Dict[Tuple[str, str], NullTranslations] = defaultdict(NullTranslations)
 
 
-def init(locale_dirs: List[Optional[str]], language: str,
+def init(locale_dirs: List[Optional[str]], language: Optional[str],
          catalog: str = 'sphinx', namespace: str = 'general') -> Tuple[NullTranslations, bool]:
     """Look for message catalogs in `locale_dirs` and *ensure* that there is at
     least a NullTranslations catalog set in `translators`. If called multiple
@@ -123,9 +123,11 @@ def init(locale_dirs: List[Optional[str]], language: str,
 
     if language and '_' in language:
         # for language having country code (like "de_AT")
-        languages = [language, language.split('_')[0]]
-    else:
+        languages: Optional[List[str]] = [language, language.split('_')[0]]
+    elif language:
         languages = [language]
+    else:
+        languages = None
 
     # loading
     for dir_ in locale_dirs:
@@ -212,7 +214,7 @@ def get_translation(catalog: str, namespace: str = 'general') -> Callable:
 
 
         def setup(app):
-            package_dir = path.abspath(path.dirname(__file__))
+            package_dir = os.path.abspath(os.path.dirname(__file__))
             locale_dir = os.path.join(package_dir, 'locales')
             app.add_message_catalog(MESSAGE_CATALOG_NAME, locale_dir)
 
@@ -259,8 +261,8 @@ admonitionlabels = {
     'warning':   _('Warning'),
 }
 
-# Moved to sphinx.directives.other (will be overriden later)
-versionlabels = {}  # type: Dict[str, str]
+# Moved to sphinx.directives.other (will be overridden later)
+versionlabels: Dict[str, str] = {}
 
-# Moved to sphinx.domains.python (will be overriden later)
-pairindextypes = {}  # type: Dict[str, str]
+# Moved to sphinx.domains.python (will be overridden later)
+pairindextypes: Dict[str, str] = {}
