@@ -7,7 +7,7 @@
 
     :author: Sebastian Wiesner
     :contact: basti.wiesner@gmx.net
-    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -16,6 +16,7 @@ import sys
 from distutils.cmd import Command
 from distutils.errors import DistutilsExecError
 from io import StringIO
+from typing import Any, Dict
 
 from sphinx.application import Sphinx
 from sphinx.cmd.build import handle_exception
@@ -23,18 +24,14 @@ from sphinx.util.console import color_terminal, nocolor
 from sphinx.util.docutils import docutils_namespace, patch_docutils
 from sphinx.util.osutil import abspath
 
-if False:
-    # For type annotation
-    from typing import Any, Dict  # NOQA
-
 
 class BuildDoc(Command):
     """
     Distutils command to build Sphinx documentation.
 
     The Sphinx build can then be triggered from distutils, and some Sphinx
-    options can be set in ``setup.py`` or ``setup.cfg`` instead of Sphinx own
-    configuration file.
+    options can be set in ``setup.py`` or ``setup.cfg`` instead of Sphinx's
+    own configuration file.
 
     For instance, from `setup.py`::
 
@@ -91,18 +88,18 @@ class BuildDoc(Command):
     boolean_options = ['fresh-env', 'all-files', 'warning-is-error',
                        'link-index', 'nitpicky']
 
-    def initialize_options(self):
-        # type: () -> None
+    def initialize_options(self) -> None:
         self.fresh_env = self.all_files = False
         self.pdb = False
-        self.source_dir = self.build_dir = None  # type: str
+        self.source_dir: str = None
+        self.build_dir: str = None
         self.builder = 'html'
         self.warning_is_error = False
         self.project = ''
         self.version = ''
         self.release = ''
         self.today = ''
-        self.config_dir = None  # type: str
+        self.config_dir: str = None
         self.link_index = False
         self.copyright = ''
         # Link verbosity to distutils' (which uses 1 by default).
@@ -111,8 +108,7 @@ class BuildDoc(Command):
         self.nitpicky = False
         self.keep_going = False
 
-    def _guess_source_dir(self):
-        # type: () -> str
+    def _guess_source_dir(self) -> str:
         for guess in ('doc', 'docs'):
             if not os.path.isdir(guess):
                 continue
@@ -121,8 +117,7 @@ class BuildDoc(Command):
                     return root
         return os.curdir
 
-    def finalize_options(self):
-        # type: () -> None
+    def finalize_options(self) -> None:
         self.ensure_string_list('builder')
 
         if self.source_dir is None:
@@ -144,15 +139,14 @@ class BuildDoc(Command):
             (builder, os.path.join(self.build_dir, builder))
             for builder in self.builder]
 
-    def run(self):
-        # type: () -> None
+    def run(self) -> None:
         if not color_terminal():
             nocolor()
         if not self.verbose:  # type: ignore
             status_stream = StringIO()
         else:
             status_stream = sys.stdout  # type: ignore
-        confoverrides = {}  # type: Dict[str, Any]
+        confoverrides: Dict[str, Any] = {}
         if self.project:
             confoverrides['project'] = self.project
         if self.version:
@@ -190,6 +184,6 @@ class BuildDoc(Command):
             if not self.link_index:
                 continue
 
-            src = app.config.master_doc + app.builder.out_suffix  # type: ignore
+            src = app.config.root_doc + app.builder.out_suffix  # type: ignore
             dst = app.builder.get_outfilename('index')  # type: ignore
             os.symlink(src, dst)
