@@ -26,12 +26,16 @@
 """
 
 import re
+import warnings
 from typing import Generator, Iterable, Tuple
 
 from docutils.utils import smartquotes
 
+from sphinx.deprecation import RemovedInSphinx60Warning
 from sphinx.util.docutils import __version_info__ as docutils_version
 
+warnings.warn('sphinx.util.smartypants is deprecated.',
+              RemovedInSphinx60Warning)
 
 langquotes = {'af':           '“”‘’',
               'af-x-altquot': '„”‚’',
@@ -320,7 +324,7 @@ def educate_tokens(text_tokens: Iterable[Tuple[str, str]],
 
     for (ttype, text) in text_tokens:
 
-        # skip HTML and/or XML tags as well as emtpy text tokens
+        # skip HTML and/or XML tags as well as empty text tokens
         # without updating the last character
         if ttype == 'tag' or not text:
             yield text
@@ -357,7 +361,7 @@ def educate_tokens(text_tokens: Iterable[Tuple[str, str]],
             text = smartquotes.educateSingleBackticks(text, language)
 
         if do_quotes:
-            # Replace plain quotes to prevent converstion to
+            # Replace plain quotes to prevent conversion to
             # 2-character sequence in French.
             context = prev_token_last_char.replace('"', ';').replace("'", ';')
             text = educateQuotes(context + text, language)[1:]
