@@ -11,8 +11,6 @@
 import re
 from typing import Dict
 
-from sphinx.deprecation import RemovedInSphinx40Warning, deprecated_alias
-
 tex_replacements = [
     # map TeX special chars
     ('$', r'\$'),
@@ -31,6 +29,8 @@ tex_replacements = [
     # map special Unicode characters to TeX commands
     ('✓', r'\(\checkmark\)'),
     ('✔', r'\(\pmb{\checkmark}\)'),
+    ('✕', r'\(\times\)'),
+    ('✖', r'\(\pmb{\times}\)'),
     # used to separate -- in options
     ('﻿', r'{}'),
     # map some special Unicode characters to similar ASCII ones
@@ -100,20 +100,12 @@ unicode_tex_replacements = [
 # %, {, }, \, #, and ~ are the only ones which must be replaced by _ character
 # It would be simpler to define it entirely here rather than in init().
 # Unicode replacements are superfluous, as idescape() uses backslashreplace
-tex_replace_map = {}  # type: Dict[int, str]
+tex_replace_map: Dict[int, str] = {}
 
-_tex_escape_map = {}  # type: Dict[int, str]
-_tex_escape_map_without_unicode = {}  # type: Dict[int, str]
-_tex_hlescape_map = {}  # type: Dict[int, str]
-_tex_hlescape_map_without_unicode = {}  # type: Dict[int, str]
-
-
-deprecated_alias('sphinx.util.texescape',
-                 {
-                     'tex_escape_map': _tex_escape_map,
-                     'tex_hl_escape_map_new': _tex_hlescape_map,
-                 },
-                 RemovedInSphinx40Warning)
+_tex_escape_map: Dict[int, str] = {}
+_tex_escape_map_without_unicode: Dict[int, str] = {}
+_tex_hlescape_map: Dict[int, str] = {}
+_tex_hlescape_map_without_unicode: Dict[int, str] = {}
 
 
 def escape(s: str, latex_engine: str = None) -> str:
