@@ -15,8 +15,6 @@ from xml.etree import ElementTree
 
 import pytest
 
-from sphinx.util import docutils
-
 
 # check given command is runnable
 def runnable(command):
@@ -348,15 +346,13 @@ def test_epub_css_files(app):
     content = (app.outdir / 'index.xhtml').read_text()
     assert '<link rel="stylesheet" type="text/css" href="_static/css/epub.css" />' in content
 
-    # files in html_css_files are not outputed
+    # files in html_css_files are not outputted
     assert ('<link rel="stylesheet" type="text/css" href="_static/css/style.css" />'
             not in content)
     assert ('<link media="print" rel="stylesheet" title="title" type="text/css" '
             'href="https://example.com/custom.css" />' not in content)
 
 
-@pytest.mark.skipif(docutils.__version_info__ < (0, 13),
-                    reason='docutils-0.13 or above is required')
 @pytest.mark.sphinx('epub', testroot='roles-download')
 def test_html_download_role(app, status, warning):
     app.build()
